package net.jxta.chat.interfaces;
import java.awt.Color;
import java.awt.ComponentOrientation;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.Enumeration;

import javax.swing.AbstractButton;
import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.border.EtchedBorder;
import javax.swing.border.SoftBevelBorder;

import net.jxta.chat.core.ChatHandler;
import net.jxta.chat.core.PeerCore;
import net.jxta.peergroup.PeerGroup;

/**
 * @author Matteo Tanca
 * @version 0.1
 * Interfaccia di chat
 */
public class ChatInterface extends JFrame {

	private static final long serialVersionUID = 1L;
	private JPanel mainPanel = null;
	
	/* 
	 * Stringa di default che segue l'identificatore del mittente nella
	 * stampa dei messaggi
	 */
	public static final String SEPARATOR = " scrive:\n";
	
	private String chatTitle = null;
	private boolean historyHasToScroll = true;
	private ChatHandler handler = null;
	private ChatInterfaceEventListener interfaceEventListener = null;
	private DefaultListModel peerListModel = null;
	private String nick;
	
	private JButton refreshUsersButton = null;
	private JButton sendMsgButton = null;
	private JLabel listLabel = null;
	private JLabel jLabel = null;
	private JMenuBar jJMenuBar = null;
	private JScrollPane jScrollPane = null;
	private JList peerList = null;
	private JCheckBox autoScrollCheckBox = null;
	private JScrollPane jScrollPane1 = null;
	private JTextArea historyArea = null;
	private JScrollPane jScrollPane2 = null;
	private JTextArea inputArea = null;
	private JMenu jMenu = null;
	private JMenuItem saveHistoryMenuItem = null;
	private JMenuItem exitMenuItem = null;
	private JMenu jMenu1 = null;
	private JMenu jMenu2 = null;
	private JMenuItem setNickname = null;
	private JRadioButtonMenuItem onlineMenuItem = null;
	private JRadioButtonMenuItem awayMenuItem = null;
	private JRadioButtonMenuItem backSoonMenuItem = null;
	private JRadioButtonMenuItem busyMenuItem = null;
	private JRadioButtonMenuItem offlineMenuItem = null;
	private ButtonGroup stats = null;
	private JMenu jMenu3 = null;
	private JCheckBoxMenuItem logChangesCheckBox = null;
	private JMenuItem changeBackgroundItem = null;
	private JMenuItem changeFontsMenuItem = null;
	private JMenuItem changeTextColorItem = null;
	private JPanel jPanel = null;
	
	public ChatInterface (PeerCore pc, PeerGroup chat, String nickName) {
		super();
		chatTitle = nickName + " @ " + chat.getPeerGroupName() + " [" + chat.getPeerGroupAdvertisement().getDescription() + "] (" + "GID: " + chat.getPeerGroupID() + ")";
		initialize();
		handler = new ChatHandler(pc, chat, this, peerListModel, nickName);
		nick = nickName;
		setVisible(true);
		interfaceEventListener = new ChatInterfaceEventListener(this, handler, inputArea);
		initializeListeners();
	}
	/**
	 * Metodo privato di inizializzazione dell'interfaccia
	 * di chat
	 */
	private void initialize() {
		this.setTitle(chatTitle);
		this.setSize(576, 494);
		this.setContentPane(getMainPanel());
		this.setJMenuBar(getJJMenuBar());
	}
	/**
	 * Metodo privato di inizializzazione del pannello principale
	 * @return JPanel mainPanel
	 */
	private JPanel getMainPanel() {
		if (mainPanel == null) {
			GridBagConstraints gridBagConstraints8 = new GridBagConstraints();
			gridBagConstraints8.insets = new Insets(1, 1, 2, 0);
			gridBagConstraints8.gridy = 1;
			gridBagConstraints8.fill = GridBagConstraints.BOTH;
			gridBagConstraints8.weightx = 1.0D;
			gridBagConstraints8.weighty = 1.0D;
			gridBagConstraints8.gridx = 1;
			GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
			gridBagConstraints7.insets = new Insets(0, 1, 0, 4);
			gridBagConstraints7.gridy = 0;
			gridBagConstraints7.ipadx = 228;
			gridBagConstraints7.ipady = 15;
			gridBagConstraints7.gridx = 1;
			GridBagConstraints gridBagConstraints6 = new GridBagConstraints();
			gridBagConstraints6.insets = new Insets(0, 0, 0, 1);
			gridBagConstraints6.gridy = 0;
			gridBagConstraints6.ipadx = 63;
			gridBagConstraints6.ipady = 15;
			gridBagConstraints6.gridx = 0;
			GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
			gridBagConstraints5.insets = new Insets(2, 1, 0, 3);
			gridBagConstraints5.gridy = 2;
			gridBagConstraints5.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints5.gridx = 0;
			GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
			gridBagConstraints4.fill = GridBagConstraints.BOTH;
			gridBagConstraints4.gridx = 0;
			gridBagConstraints4.gridy = 1;
			gridBagConstraints4.ipadx = -91;
			gridBagConstraints4.ipady = 252;
			gridBagConstraints4.weightx = 0.15D;
			gridBagConstraints4.weighty = 1.0;
			gridBagConstraints4.insets = new Insets(0, 0, 1, 1);
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			gridBagConstraints3.insets = new Insets(2, 8, 0, 7);
			gridBagConstraints3.gridy = 2;
			gridBagConstraints3.ipadx = 238;
			gridBagConstraints3.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints3.gridx = 1;
			mainPanel = new JPanel();
			listLabel = new JLabel();
			listLabel.setHorizontalAlignment(SwingConstants.CENTER);
			listLabel.setText("Stato degli utenti");
			jLabel = new JLabel();
			jLabel.setHorizontalAlignment(SwingConstants.CENTER);
			jLabel.setText("Cronologia dei messaggi");
			mainPanel.setLayout(new GridBagLayout());
			mainPanel.setBorder(new SoftBevelBorder(SoftBevelBorder.RAISED));
			mainPanel.add(getSendMsgButton(), gridBagConstraints3);
			mainPanel.add(getJScrollPane(), gridBagConstraints4);
			mainPanel.add(getRefreshUsersButton(), gridBagConstraints5);
			mainPanel.add(listLabel, gridBagConstraints6);
			mainPanel.add(jLabel, gridBagConstraints7);
			mainPanel.add(getJPanel(), gridBagConstraints8);
		}
		return mainPanel;
	}
	/**
	 * Metodo privato di inizializzazione del bottone 'Aggiorna	stato utenti'
	 * @return JButton refreshUsersButton
	 */
	private JButton getRefreshUsersButton() {
		if (refreshUsersButton == null) {
			refreshUsersButton = new JButton();
			refreshUsersButton.setToolTipText("Clicca per richiedere un aggiornamento forzato dello stato degli utenti");
			refreshUsersButton.setName("Refresh");
			refreshUsersButton.setText("Aggiorna stato utenti");
		}
		return refreshUsersButton;
	}
	/**
	 * Metodo privato di inizializzazione del bottone 'Invia messaggio'
	 * @return JButton sendMsgButton
	 */
	private JButton getSendMsgButton() {
		if (sendMsgButton == null) {
			sendMsgButton = new JButton();
			sendMsgButton.setToolTipText("Premi per inviare il messaggio digitato");
			sendMsgButton.setName("SendMsg");
			sendMsgButton.setText("Invia messaggio");
		}
		return sendMsgButton;
	}

	/**
	 * Metodo privato di inizializzazione della barra del menu'
	 * @return JMenuBar	
	 */
	private JMenuBar getJJMenuBar() {
		if (jJMenuBar == null) {
			jJMenuBar = new JMenuBar();
			jJMenuBar.setPreferredSize(new Dimension(0, 20));
			stats = new ButtonGroup();
			jJMenuBar.add(getJMenu());
			jJMenuBar.add(getJMenu1());
			jJMenuBar.add(getJMenu2());
			jJMenuBar.add(getJMenu3());
		}
		return jJMenuBar;
	}
	/**
	 * Metodo privato di inizializzazione del pannello di scorrimento in cui
	 * e' inserita la lista degli utenti
	 * @return JScrollPane	
	 */
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createEtchedBorder(EtchedBorder.RAISED), BorderFactory.createLineBorder(Color.black, 1)));
			jScrollPane.setViewportView(getPeerList());
		}
		return jScrollPane;
	}
	/**
	 * Metodo privato di inizializzazione della lista degli utenti
	 * @return JList peerList
	 */
	private JList getPeerList() {
		if (peerList == null) {
			peerListModel = new DefaultListModel();
			peerList = new JList(peerListModel);
			peerList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		}
		return peerList;
	}
	/**
	 * Metodo privato di inizializzazione della check box
	 * @return JCheckBox autoScrollCheckBox
	 */
	private JCheckBox getAutoScrollCheckBox() {
		if (autoScrollCheckBox == null) {
			autoScrollCheckBox = new JCheckBox();
			autoScrollCheckBox.setToolTipText("Utile in fase di consultazione per evitare lo scorrimento verso la fine della cronologia all'arrivo di nuovi messaggi");
			autoScrollCheckBox.setComponentOrientation(ComponentOrientation.LEFT_TO_RIGHT);
			autoScrollCheckBox.setHorizontalAlignment(SwingConstants.LEFT);
			autoScrollCheckBox.setText("Disabilita l'autoscroll della cronologia");
		}
		return autoScrollCheckBox;
	}
	/**
	 * Metodo privato di inizializzazione del pannello di scorrimento in cui e'
	 * inserita l'area di testo contenente la cronologia
	 * @return JScrollPane	
	 */
	private JScrollPane getJScrollPane1() {
		if (jScrollPane1 == null) {
			jScrollPane1 = new JScrollPane();
			jScrollPane1.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
			jScrollPane1.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createEtchedBorder(EtchedBorder.RAISED), BorderFactory.createLineBorder(Color.black, 1)));
			jScrollPane1.setViewportView(getHistoryArea());
		}
		return jScrollPane1;
	}
	/**
	 * Metodo privato di inizializzazione della JTextArea contenente la cronologia
	 * @return JTextArea historyArea
	 */
	private JTextArea getHistoryArea() {
		if (historyArea == null) {
			historyArea = new JTextArea();
			historyArea.setEditable(false);
			historyArea.setLineWrap(true);
			historyArea.setWrapStyleWord(true);
			historyArea.setBackground(Color.WHITE);
			historyArea.setForeground(Color.BLACK);
			
		}
		return historyArea;
	}
	/**
	 * Metodo privato di inizializzazione del pannello di scorrimento in cui e'
	 * inserita l'area di testo di input
	 * @return JScrollPane	
	 */
	private JScrollPane getJScrollPane2() {
		if (jScrollPane2 == null) {
			jScrollPane2 = new JScrollPane();
			jScrollPane2.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createEtchedBorder(EtchedBorder.RAISED), BorderFactory.createLineBorder(Color.black, 1)));
			jScrollPane2.setPreferredSize(new Dimension(424, 24));
			jScrollPane2.setViewportView(getInputArea());
		}
		return jScrollPane2;
	}
	/**
	 * Metodo privato di inizializzazione della JTextArea per l'input
	 * @return JTextArea inputArea
	 */
	private JTextArea getInputArea() {
		if (inputArea == null) {
			inputArea = new JTextArea();
			inputArea.setLineWrap(true);
			inputArea.setSize(new Dimension(385, 137));
			inputArea.setWrapStyleWord(true);
			inputArea.setBackground(Color.WHITE);
			inputArea.setForeground(Color.BLACK);
		}
		return inputArea;
	}
	/**
	 * Metodo chiamato dal ChatInterfaceEventListener dopo l'inserimento
	 * del nuovo nick/stato
	 * @param nickName
	 */
	protected void refreshTitle (String nickName) {
		chatTitle = chatTitle.substring(nick.length());
		nick = nickName;
		chatTitle = nick + chatTitle;
		this.setTitle(chatTitle);
	}
	
	/**
	 * Metodo privato di inizializzazione dei listener degli eventi
	 * proveniente dalle varie componenti dell'interfaccia
	 */
	private void initializeListeners() {
		autoScrollCheckBox.addItemListener(interfaceEventListener);
		inputArea.addKeyListener(interfaceEventListener);
		refreshUsersButton.addMouseListener(interfaceEventListener);
		sendMsgButton.addMouseListener(interfaceEventListener);
		saveHistoryMenuItem.addMouseListener(interfaceEventListener);
		exitMenuItem.addMouseListener(interfaceEventListener);
		Enumeration<AbstractButton> radioButtons = stats.getElements();
		while (radioButtons.hasMoreElements())
			radioButtons.nextElement().addMouseListener(interfaceEventListener);
		setNickname.addMouseListener(interfaceEventListener);
		this.addWindowListener(interfaceEventListener);
		changeFontsMenuItem.addMouseListener(interfaceEventListener);
		logChangesCheckBox.addMouseListener(interfaceEventListener);
		changeBackgroundItem.addMouseListener(interfaceEventListener);
		changeTextColorItem.addMouseListener(interfaceEventListener);
	}

	/**
	 * Metodo che semplicemente inverte in flag di scorrimento per la cronologia
	 */
	public void invertHistoryScrollFlag () {
		historyHasToScroll = !historyHasToScroll;
	}
	/**
	 * Metodo che aggiunge in coda all'area di testo la stringa messaggio parametro
	 * E' necessaria la sincronizzazione per possibili race condition fra l'arrivo
	 * dei messaggi e l'aggiornamento della cronologia con i messaggi spediti
	 * @param msg Stringa da aggiungere in coda all'area di testo
	 */
	public void appendToHistory (String msg) {
		int caretPositionBeforeAppend = historyArea.getCaretPosition();
		historyArea.append(msg);
		if (historyHasToScroll)
			historyArea.setCaretPosition(historyArea.getDocument().getLength());
		else
			historyArea.setCaretPosition(caretPositionBeforeAppend);
	}
	
	/**
	 * This method initializes jMenu	
	 * 	
	 * @return javax.swing.JMenu	
	 */
	private JMenu getJMenu() {
		if (jMenu == null) {
			jMenu = new JMenu();
			jMenu.setName("FileMenu");
			jMenu.setText("File");
			jMenu.add(getSaveHistoryMenuItem());
			jMenu.add(getExitMenuItem());
		}
		return jMenu;
	}
	/**
	 * This method initializes saveHistoryMenuItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getSaveHistoryMenuItem() {
		if (saveHistoryMenuItem == null) {
			saveHistoryMenuItem = new JMenuItem();
			saveHistoryMenuItem.setName("SaveHistory");
			saveHistoryMenuItem.setText("Salva cronologia");
		}
		return saveHistoryMenuItem;
	}
	/**
	 * This method initializes exitMenuItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getExitMenuItem() {
		if (exitMenuItem == null) {
			exitMenuItem = new JMenuItem();
			exitMenuItem.setName("Exit");
			exitMenuItem.setText("Esci");
		}
		return exitMenuItem;
	}
	/**
	 * This method initializes jMenu1	
	 * 	
	 * @return javax.swing.JMenu	
	 */
	private JMenu getJMenu1() {
		if (jMenu1 == null) {
			jMenu1 = new JMenu();
			jMenu1.setText("Stato utente");
			jMenu1.setName("StatusMenu");
			jMenu1.add(getOnlineMenuItem());
			jMenu1.add(getAwayMenuItem());
			jMenu1.add(getBackSoonMenuItem());
			jMenu1.add(getBusyMenuItem());
			jMenu1.add(getOfflineMenuItem());
		}
		return jMenu1;
	}
	/**
	 * This method initializes jMenu2	
	 * 	
	 * @return javax.swing.JMenu	
	 */
	private JMenu getJMenu2() {
		if (jMenu2 == null) {
			jMenu2 = new JMenu();
			jMenu2.setText("Nickname");
			jMenu2.setName("NickMenu");
			jMenu2.add(getSetNickname());
		}
		return jMenu2;
	}
	/**
	 * This method initializes setNickname	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getSetNickname() {
		if (setNickname == null) {
			setNickname = new JMenuItem();
			setNickname.setName("SetNickname");
			setNickname.setText("Modifica");
		}
		return setNickname;
	}
	/**
	 * This method initializes onlineMenuItem	
	 * 	
	 * @return javax.swing.JRadioButtonMenuItem	
	 */
	private JRadioButtonMenuItem getOnlineMenuItem() {
		if (onlineMenuItem == null) {
			onlineMenuItem = new JRadioButtonMenuItem();
			onlineMenuItem.setName("OnlineItem");
			onlineMenuItem.setText("In linea");
			onlineMenuItem.setSelected(true);
			stats.add(onlineMenuItem);
		}
		return onlineMenuItem;
	}
	/**
	 * This method initializes awayMenuItem	
	 * 	
	 * @return javax.swing.JRadioButtonMenuItem	
	 */
	private JRadioButtonMenuItem getAwayMenuItem() {
		if (awayMenuItem == null) {
			awayMenuItem = new JRadioButtonMenuItem();
			awayMenuItem.setName("AwayItem");
			awayMenuItem.setText("Non al computer");
			stats.add(awayMenuItem);
		}
		return awayMenuItem;
	}
	/**
	 * This method initializes backSoonMenuItem	
	 * 	
	 * @return javax.swing.JRadioButtonMenuItem	
	 */
	private JRadioButtonMenuItem getBackSoonMenuItem() {
		if (backSoonMenuItem == null) {
			backSoonMenuItem = new JRadioButtonMenuItem();
			backSoonMenuItem.setName("BackSoonItem");
			backSoonMenuItem.setText("Torno subito");
			stats.add(backSoonMenuItem);
		}
		return backSoonMenuItem;
	}
	/**
	 * This method initializes busyMenuItem	
	 * 	
	 * @return javax.swing.JRadioButtonMenuItem	
	 */
	private JRadioButtonMenuItem getBusyMenuItem() {
		if (busyMenuItem == null) {
			busyMenuItem = new JRadioButtonMenuItem();
			busyMenuItem.setName("BusyItem");
			busyMenuItem.setText("Occupato");
			stats.add(busyMenuItem);
		}
		return busyMenuItem;
	}
	/**
	 * This method initializes offlineMenuItem	
	 * 	
	 * @return javax.swing.JRadioButtonMenuItem	
	 */
	private JRadioButtonMenuItem getOfflineMenuItem() {
		if (offlineMenuItem == null) {
			offlineMenuItem = new JRadioButtonMenuItem();
			offlineMenuItem.setName("OfflineItem");
			offlineMenuItem.setText("Non in linea");
			stats.add(offlineMenuItem);
		}
		return offlineMenuItem;
	}
	/**
	 * This method initializes jMenu3	
	 * 	
	 * @return javax.swing.JMenu	
	 */
	private JMenu getJMenu3() {
		if (jMenu3 == null) {
			jMenu3 = new JMenu();
			jMenu3.setText("Opzioni");
			jMenu3.add(getChangeBackgroundItem());
			jMenu3.add(getChangeTextColorItem());
			jMenu3.add(getChangeFontsMenuItem());
			jMenu3.addSeparator();
			jMenu3.add(getLogChangesCheckBox());
		}
		return jMenu3;
	}
	/**
	 * This method initializes logChangesCheckBox	
	 * 	
	 * @return javax.swing.JCheckBoxMenuItem	
	 */
	private JCheckBoxMenuItem getLogChangesCheckBox() {
		if (logChangesCheckBox == null) {
			logChangesCheckBox = new JCheckBoxMenuItem();
			logChangesCheckBox.setName("LogCheckBox");
			logChangesCheckBox.setText("Registra cambi di stato degli utenti nella cronologia");
			logChangesCheckBox.setSelected(true);
		}
		return logChangesCheckBox;
	}
	/**
	 * This method initializes changeBackgroundItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getChangeBackgroundItem() {
		if (changeBackgroundItem == null) {
			changeBackgroundItem = new JMenuItem();
			changeBackgroundItem.setName("ChangeBackgroundItem");
			changeBackgroundItem.setActionCommand("");
			changeBackgroundItem.setText("Colore sfondo");
		}
		return changeBackgroundItem;
	}
	/**
	 * This method initializes changeFontsMenuItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getChangeFontsMenuItem() {
		if (changeFontsMenuItem == null) {
			changeFontsMenuItem = new JMenuItem();
			changeFontsMenuItem.setName("ChangeTextItem");
			changeFontsMenuItem.setText("Font");
		}
		return changeFontsMenuItem;
	}
	/**
	 * Metodo di impostazione nuovo Font per le aree testo
	 * @param newFont
	 */
	public void setCurrentFont (Font newFont) {
		historyArea.setFont(newFont);
		inputArea.setFont(newFont);
	}
	
	/**
	 * Metodo che restituisce il testo contenuto nell'history
	 * @return String Cronologia dei messaggi
	 */
	public String getHistory () {
		return historyArea.getText();
	}

	/**
	 * Metodo che restituisce il colore utilizzato nelle finestre di
	 * testo
	 * @return Color
	 */
	public Color getTextColor() {
		return historyArea.getForeground();
	}
	
	/**
	 * Metodo che restituisce il colore di sfondo utilizzato nelle finestre di
	 * testo
	 * @return Color
	 */
	public Color getTextBackground() {
		return historyArea.getBackground();
	}
	
	/**
	 * Metodo che imposta il colore di sfondo utilizzato nelle finestre di
	 * testo
	 * @param Color Il colore che verrà utilizzato
	 */
	public void setTextBackground(Color background) {
		historyArea.setBackground(background);
		inputArea.setBackground(background);
	}
	
	/**
	 * Metodo che imposta il colore utilizzato nelle finestre di
	 * testo
	 * @param Color Il colore che verrà utilizzato
	 */
	public void setTextColor (Color foreground) {
		historyArea.setForeground(foreground);
		inputArea.setForeground(foreground);
	}
	/**
	 * Metodo che restituisce il font corrente
	 * @return Font Il font attualmente in uso in aree testo
	 */
	public Font getTextFont() {
		return historyArea.getFont();
	}
	/**
	 * This method initializes changeTextColorItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getChangeTextColorItem() {
		if (changeTextColorItem == null) {
			changeTextColorItem = new JMenuItem();
			changeTextColorItem.setText("Colore testo");
			changeTextColorItem.setName("ChangeTextColorItem");
		}
		return changeTextColorItem;
	}
	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanel() {
		if (jPanel == null) {
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.fill = GridBagConstraints.BOTH;
			gridBagConstraints2.gridx = 0;
			gridBagConstraints2.gridy = 2;
			gridBagConstraints2.ipady = 110;
			gridBagConstraints2.weightx = 1.0;
			gridBagConstraints2.weighty = 1.0;
			gridBagConstraints2.insets = new Insets(0, 0, -1, 2);
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.insets = new Insets(1, 1, 0, 1);
			gridBagConstraints1.gridy = 1;
			gridBagConstraints1.ipadx = 124;
			gridBagConstraints1.ipady = -4;
			gridBagConstraints1.weightx = 1.0D;
			gridBagConstraints1.weighty = 0.05D;
			gridBagConstraints1.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints1.anchor = GridBagConstraints.WEST;
			gridBagConstraints1.gridx = 0;
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.fill = GridBagConstraints.BOTH;
			gridBagConstraints.gridx = 0;
			gridBagConstraints.gridy = 0;
			gridBagConstraints.ipady = 181;
			gridBagConstraints.weightx = 1.0;
			gridBagConstraints.weighty = 1.0;
			gridBagConstraints.insets = new Insets(0, 0, 0, 2);
			jPanel = new JPanel();
			jPanel.setLayout(new GridBagLayout());
			jPanel.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
			jPanel.add(getJScrollPane1(), gridBagConstraints);
			jPanel.add(getAutoScrollCheckBox(), gridBagConstraints1);
			jPanel.add(getJScrollPane2(), gridBagConstraints2);
		}
		return jPanel;
	}
}  //  @jve:decl-index=0:visual-constraint="10,-6"
