package net.jxta.chat.interfaces;
import javax.swing.JPanel;
import java.awt.Frame;
import javax.swing.JDialog;
import javax.swing.BorderFactory;
import javax.swing.border.EtchedBorder;
import java.awt.Color;
import javax.swing.border.TitledBorder;
import java.awt.Font;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JButton;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;


/**
 * @author Matteo Tanca
 * @version 0.1
 * Classe di supporto che implementa una finestra di dialogo personalizzata
 * per l'inserimento delle informazioni relative alla creazione di una nuova
 * chat
 */
public class CreateChatDialog extends JDialog {

	private static final long serialVersionUID = 1L;
	private JPanel jContentPane = null;
	private JLabel nameFieldLabel = null;
	private JTextField nameField = null;
	private JLabel descriptionFieldLabel = null;
	private JTextField descriptionField = null;
	private JButton confirmButton = null;
	private JButton cancelButton = null;
	
	private String chatName, chatDescription;
	private boolean inputConfirmed;
	
	public CreateChatDialog (Frame owner) {
		super(owner, true);
		inputConfirmed = false;
		initialize();
		setLocationRelativeTo(owner);
	}
	
	/**
	 * Metodo di inizializzazione della finestra di dialogo
	 */
	private void initialize() {
		this.setSize(300, 200);
		this.setModal(true);
		this.setResizable(false);
		this.setTitle("Creazione chat");
		this.setContentPane(getJContentPane());
		ActionListener clickListener = new ActionListener() {
			public void actionPerformed (ActionEvent e) {
				chatName = nameField.getText();
				chatDescription = descriptionField.getText();
				inputConfirmed = true;
				dispose();
			}
		};
		confirmButton.addActionListener(clickListener);
		clickListener = new ActionListener() {
			public void actionPerformed (ActionEvent e) {
				inputConfirmed = false;
				dispose();
			}
		};
		cancelButton.addActionListener(clickListener);
	}
	
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
			gridBagConstraints5.insets = new Insets(6, 10, 10, 38);
			gridBagConstraints5.gridy = 3;
			gridBagConstraints5.ipadx = 10;
			gridBagConstraints5.ipady = 12;
			gridBagConstraints5.gridx = 1;
			GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
			gridBagConstraints4.insets = new Insets(6, 24, 10, 9);
			gridBagConstraints4.gridy = 3;
			gridBagConstraints4.ipadx = 5;
			gridBagConstraints4.ipady = 12;
			gridBagConstraints4.gridx = 0;
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			gridBagConstraints3.fill = GridBagConstraints.VERTICAL;
			gridBagConstraints3.gridwidth = 2;
			gridBagConstraints3.gridx = 0;
			gridBagConstraints3.gridy = 2;
			gridBagConstraints3.ipadx = 272;
			gridBagConstraints3.ipady = 4;
			gridBagConstraints3.weightx = 1.0;
			gridBagConstraints3.insets = new Insets(22, 1, 5, 0);
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.insets = new Insets(2, 2, 27, 22);
			gridBagConstraints2.gridy = 2;
			gridBagConstraints2.ipadx = 36;
			gridBagConstraints2.ipady = 6;
			gridBagConstraints2.gridx = 0;
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.fill = GridBagConstraints.VERTICAL;
			gridBagConstraints1.gridwidth = 2;
			gridBagConstraints1.gridx = 0;
			gridBagConstraints1.gridy = 1;
			gridBagConstraints1.ipadx = 272;
			gridBagConstraints1.ipady = 4;
			gridBagConstraints1.weightx = 1.0;
			gridBagConstraints1.insets = new Insets(0, 0, 2, 1);
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.insets = new Insets(0, 3, 0, 22);
			gridBagConstraints.gridy = 0;
			gridBagConstraints.ipadx = 7;
			gridBagConstraints.ipady = 5;
			gridBagConstraints.gridx = 0;
			descriptionFieldLabel = new JLabel();
			descriptionFieldLabel.setText("Descrizione");
			nameFieldLabel = new JLabel();
			nameFieldLabel.setText("Titolo della chat");
			jContentPane = new JPanel();
			jContentPane.setLayout(new GridBagLayout());
			jContentPane.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createEtchedBorder(EtchedBorder.RAISED), BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.lightGray, 2), "Inserire le informazioni sulla nuova chat", TitledBorder.DEFAULT_JUSTIFICATION, TitledBorder.DEFAULT_POSITION, new Font("Dialog", Font.BOLD | Font.ITALIC, 12), Color.darkGray)));
			jContentPane.setBackground(new Color(204, 204, 255));
			jContentPane.add(nameFieldLabel, gridBagConstraints);
			jContentPane.add(getNameField(), gridBagConstraints1);
			jContentPane.add(descriptionFieldLabel, gridBagConstraints2);
			jContentPane.add(getDescriptionField(), gridBagConstraints3);
			jContentPane.add(getConfirmButton(), gridBagConstraints4);
			jContentPane.add(getCancelButton(), gridBagConstraints5);
		}
		return jContentPane;
	}
	
	private JTextField getNameField() {
		if (nameField == null) {
			nameField = new JTextField();
		}
		return nameField;
	}
	
	private JTextField getDescriptionField() {
		if (descriptionField == null) {
			descriptionField = new JTextField();
		}
		return descriptionField;
	}

	private JButton getConfirmButton() {
		if (confirmButton == null) {
			confirmButton = new JButton();
			confirmButton.setName("confirmButton");
			confirmButton.setMnemonic(KeyEvent.VK_UNDEFINED);
			confirmButton.setText("Conferma");
		}
		return confirmButton;
	}
	
	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
			cancelButton.setText("Cancella");
			cancelButton.setName("cancelButton");
		}
		return cancelButton;
	}
	
	/**
	 * Metodo che restituisce il flag di validità dell'operazione di inserimento
	 * @return boolean Flag
	 */
	public boolean isInputConfirmed () {
		return inputConfirmed;
	}
	
	/**
	 * Metodo che restituisce la stringa acquisita dal campo chatName
	 * @return String Il nome prescelto per la chat
	 */
	public String getChatName () {
		return chatName;
	}
	
	/**
	 * Metodo che restituisce la stringa acquisita dal campo chatDescription
	 * @return String La descrizione prescelta per la chat
	 */
	public String getChatDescription () {
		return chatDescription;
	}
}
