package net.jxta.chat.interfaces;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GraphicsEnvironment;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ListSelectionModel;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

/**
 * @author Matteo Tanca [tanca@cli.di.unipi.it]
 * @version 0.1
 * Semplice Dialog per la scelta del font e della dimensione del testo
 */
public class FontDialog extends JDialog {

	private static final long serialVersionUID = 1L;
	private JPanel jContentPane = null;
	private JCheckBox boldCheckBox = null;
	private JButton confirmButton = null;
	private JButton cancelButton = null;
	private JCheckBox italicCheckBox = null;
	private JPanel jPanel = null;
	private JScrollPane jScrollPane = null;
	private JScrollPane jScrollPane1 = null;
	private JTextArea textBox = null;
	private JList fontList = null;
	private JList sizeList = null;
	
	private boolean inputConfirmed;
	
	/* Possibili dimensioni dei caratteri */
	private final String[] sizes = new String[] { "2","4","6","8","10","12","14","16","20","22","24","30","36","48" };
	private JScrollPane jScrollPane2 = null;
	
	public FontDialog(Frame owner) {
		super(owner);
		initialize();
	}

	private void initialize() {
		this.setSize(331, 251);
		this.setResizable(false);
		this.setTitle("Scegli font e dimensione dei caratteri");
		this.setName("FontDialog");
		this.setModal(true);
		this.setContentPane(getJContentPane());
		ActionListener clickListener = new ActionListener() {
			public void actionPerformed (ActionEvent e) {
				
				inputConfirmed = true;
				dispose();
			}
		};
		confirmButton.addActionListener(clickListener);
		clickListener = new ActionListener() {
			public void actionPerformed (ActionEvent e) {
				inputConfirmed = false;
				dispose();
			}
		};
		cancelButton.addActionListener(clickListener);
		
		ListSelectionListener listListener = new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				textBox.setFont(getCurrentFont());
			}
		};

		fontList.addListSelectionListener(listListener);
		sizeList.addListSelectionListener(listListener);


		clickListener = new ActionListener() {
			public void actionPerformed(ActionEvent e){ 
				textBox.setFont(getCurrentFont());
			}
		};
		
		boldCheckBox.addActionListener(clickListener);
		italicCheckBox.addActionListener(clickListener);
		
		setCurrentFont(null);
		
	}

	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(null);
			jContentPane.add(getConfirmButton(), null);
			jContentPane.add(getCancelButton(), null);
			jContentPane.add(getJPanel(), null);
			jContentPane.add(getJScrollPane(), null);
			jContentPane.add(getJScrollPane1(), null);
		}
		return jContentPane;
	}

	/**
	 * This method initializes boldCheckBox	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */
	private JCheckBox getBoldCheckBox() {
		if (boldCheckBox == null) {
			boldCheckBox = new JCheckBox();
			boldCheckBox.setName("BoldCheckBox");
			boldCheckBox.setText("Grassetto");
			boldCheckBox.setSize(new Dimension(85, 23));
			boldCheckBox.setLocation(new Point(2, 2));
			boldCheckBox.setEnabled(true);
		}
		return boldCheckBox;
	}

	/**
	 * This method initializes confirmButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getConfirmButton() {
		if (confirmButton == null) {
			confirmButton = new JButton();
			confirmButton.setName("OK");
			confirmButton.setSize(new Dimension(135, 34));
			confirmButton.setLocation(new Point(20, 189));
			confirmButton.setText("OK");
		}
		return confirmButton;
	}

	/**
	 * This method initializes cancelButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
			cancelButton.setName("Cancel");
			cancelButton.setSize(new Dimension(132, 34));
			cancelButton.setLocation(new Point(172, 189));
			cancelButton.setText("Cancella");
		}
		return cancelButton;
	}

	/**
	 * This method initializes italicCheckBox	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */
	private JCheckBox getItalicCheckBox() {
		if (italicCheckBox == null) {
			italicCheckBox = new JCheckBox();
			italicCheckBox.setName("ItalicCheckBox");
			italicCheckBox.setSize(new Dimension(73, 23));
			italicCheckBox.setLocation(new Point(2, 20));
			italicCheckBox.setText("Corsivo");
		}
		return italicCheckBox;
	}

	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanel() {
		if (jPanel == null) {
			jPanel = new JPanel();
			jPanel.setLayout(null);
			jPanel.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.RAISED));
			jPanel.setBounds(new Rectangle(0, 149, 321, 78));
			jPanel.add(getBoldCheckBox(), null);
			jPanel.add(getItalicCheckBox(), null);
			jPanel.add(getJScrollPane2(), null);
		}
		return jPanel;
	}

	/**
	 * This method initializes jScrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setBounds(new Rectangle(0, 0, 168, 148));
			jScrollPane.setViewportView(getFontList());
		}
		return jScrollPane;
	}

	/**
	 * This method initializes jScrollPane1	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getJScrollPane1() {
		if (jScrollPane1 == null) {
			jScrollPane1 = new JScrollPane();
			jScrollPane1.setBounds(new Rectangle(169, 0, 154, 148));
			jScrollPane1.setViewportView(getSizeList());
		}
		return jScrollPane1;
	}

	/**
	 * This method initializes textBox	
	 * 	
	 * @return javax.swing.JTextArea	
	 */
	private JTextArea getTextBox() {
		if (textBox == null) {
			textBox = new JTextArea();
			textBox.setText("The quick brown fox");
			textBox.setEditable(false);
			textBox.setRows(2);
			textBox.setLineWrap(true);
			textBox.setWrapStyleWord(true);
			textBox.setName("Preview");
		}
		return textBox;
	}

	/**
	 * This method initializes fontList	
	 * 	
	 * @return javax.swing.JList	
	 */
	private JList getFontList() {
		if (fontList == null) {
			fontList = new JList(GraphicsEnvironment.getLocalGraphicsEnvironment().
					 getAvailableFontFamilyNames());
			fontList.setName("FontList");
			fontList.setSelectedIndex(0);
			fontList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		}
		return fontList;
	}

	/**
	 * This method initializes sizeList	
	 * 	
	 * @return javax.swing.JList	
	 */
	private JList getSizeList() {
		if (sizeList == null) {
			sizeList = new JList(sizes);
			sizeList.setName("SizeList");
			sizeList.setToolTipText("Lista delle possibili dimensioni di carattere");
			sizeList.setSelectedIndex(0);
			sizeList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		}
		return sizeList;
	}
	
	private void setCurrentFont (Font font) {
		if (font == null)
			font = textBox.getFont();

		fontList.setSelectedValue(font.getName(), true);
		fontList.ensureIndexIsVisible(fontList.getSelectedIndex()); 
		sizeList.setSelectedValue("" + font.getSize(), true);
		sizeList.ensureIndexIsVisible(sizeList.getSelectedIndex());
		
		boldCheckBox.setSelected(font.isBold());
		italicCheckBox.setSelected(font.isItalic());
	}
	
	public Font getCurrentFont () {
		String family = (String)fontList.getSelectedValue();
		int size = Integer.parseInt((String)sizeList.getSelectedValue());

		int type = Font.PLAIN;

		if (boldCheckBox.isSelected()) type += Font.BOLD;
		if (italicCheckBox.isSelected()) type += Font.ITALIC;

		return new Font(family, type, size);
	}
	
	public boolean isInputConfirmed () {
		return inputConfirmed;
	}

	/**
	 * This method initializes jScrollPane2	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getJScrollPane2() {
		if (jScrollPane2 == null) {
			jScrollPane2 = new JScrollPane();
			jScrollPane2.setBounds(new Rectangle(97, 3, 224, 36));
			jScrollPane2.setViewportView(getTextBox());
		}
		return jScrollPane2;
	}
	
}  //  @jve:decl-index=0:visual-constraint="18,7"
