package net.jxta.chat.interfaces;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.io.OutputStream;
import java.io.PrintStream;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.border.EtchedBorder;
import javax.swing.border.SoftBevelBorder;

import net.jxta.chat.core.PeerCore;

/**
 * @author Matteo Tanca
 * @version 0.1
 * Interfaccia principale dell'applicazione di chat
 *
 */
public class MainInterface extends JFrame {

	private static final long serialVersionUID = 1L;
	private JPanel mainPanel = null;
	private JMenuBar barraMenu = null;
	private JMenu chatMenu = null;
	private JMenu esciMenu = null;
	private JMenu infoMenu = null;
	private JMenuItem createChatItem = null;
	private JMenuItem joinChatItem = null;
	private JMenuItem refreshItem = null;
	
	private MainInterfaceListener mainListener = null;
	private PeerCore core = null;
	private JScrollPane jScrollPane = null;
	private JList chatList = null;
	private DefaultListModel listModel = null;
	private JLabel etichetta = null;
	private JButton createChatButton = null;
	private JButton joinChatButton = null;
	private JButton refreshButton = null;
	private JTextArea statusBox = null;
	private JLabel applicationStatusLabel = null;
	private JScrollPane jScrollPane1 = null;
	private JPanel bottonPanel = null;
	private JScrollBar statusScrollBar = null;
	/**
	 * Costruttore di default
	 * Inizializza il frame e vi inserisce il mainPanel, mediante invocazione del
	 * metodo initialize
	 * @return MainInterface L'oggetto appena generato
	 */
	public MainInterface() {
		super();
		core = new PeerCore();
		initialize();
		setVisible(true);
		if (!core.initialize()) {
			String message = PeerCore.APP_NAME + " " + PeerCore.APP_VERSION + "\n" +
			"Fallita l'inizializzazione del nucleo dell'applicazione";
			JOptionPane.showMessageDialog(null, message, "Errore nell'inizializzazione del nucleo", JOptionPane.ERROR_MESSAGE);
			System.exit(1);
		}
		mainListener = new MainInterfaceListener(this, core);
		core.getChatFinder().setListModel(listModel);
		initializeListeners();
	}
		
	/**
	 * Metodo di inizializzazione del pannello di scorrimento per la lista	
	 * @return JScrollPane	
	 */
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setViewportView(getChatList());
		}
		return jScrollPane;
	}

	/**
	 * Metodo di inizializzazione della lista
	 * Sfrutta un modello, che sara' poi utilizzato dall'istanza di ChatFinder
	 * per l'aggiornamento della lista (l'aggiornamento e' realizzato tramite
	 * l'interfaccia ListDataListener, come raccomandato dalle specifiche swing)
	 * @return JList La lista delle chat disponibili
	 */
	private JList getChatList() {
		if (chatList == null) {
			listModel = new DefaultListModel();
			chatList = new JList(listModel);
			chatList.setName("ChatList");
			chatList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		}
		return chatList;
	}

	/**
	 * Metodo di inizializzazione del bottone 'Crea chat'
	 * @return JButton Il bottone 'Crea chat'
	 */
	private JButton getCreateChatButton() {
		if (createChatButton == null) {
			createChatButton = new JButton();
			createChatButton.setText("Crea chat");
			createChatButton.setToolTipText("Crea una nuova chat");
			createChatButton.setName("CreaChat");
		}
		return createChatButton;
	}
	
	/**
	 * Metodo di inizializzazione del bottone 'Partecipa alla chat'
	 * @return JButton Il bottone 'Partecipa alla chat'
	 */
	private JButton getJoinChatButton() {
		if (joinChatButton == null) {
			joinChatButton = new JButton();
			joinChatButton.setText("Partecipa alla chat");
			joinChatButton.setName("JoinChat");
			joinChatButton.setToolTipText("Partecipa alla chat selezionata dalla lista");
		}
		return joinChatButton;
	}
	
	/**
	 * Metodo di inizializzazione del bottone 'Aggiorna la lista' 
	 * @return JButton Il bottone 'Aggiorna la lista'
	 */
	private JButton getRefreshButton() {
		if (refreshButton == null) {
			refreshButton = new JButton();
			refreshButton.setText("Aggiorna la lista");
			refreshButton.setName("Refresh");
			refreshButton.setToolTipText("Aggiorna la lista delle chat");
		}
		return refreshButton;
	}

	/**
	 * This method initializes statusBox	
	 * 	
	 * @return javax.swing.JTextArea	
	 */
	private JTextArea getStatusBox() {
		if (statusBox == null) {
			statusBox = new JTextArea();
			statusBox.setName("statusBox");
			statusBox.setBackground(Color.darkGray);
			statusBox.setForeground(Color.white);
			statusBox.setEditable(false);
			statusBox.setLineWrap(true);
			statusBox.setWrapStyleWord(true);
		}
		System.setOut(new PrintStream(new AreaOutputStream(statusBox)));
		System.setErr(new PrintStream(new AreaOutputStream(statusBox)));
		return statusBox;
	}

	/**
	 * This method initializes jScrollPane1	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getJScrollPane1() {
		if (jScrollPane1 == null) {
			jScrollPane1 = new JScrollPane();
			jScrollPane1.setBorder(BorderFactory.createCompoundBorder(new SoftBevelBorder(SoftBevelBorder.LOWERED), BorderFactory.createLineBorder(Color.blue, 2)));
			jScrollPane1.setViewportView(getStatusBox());
			statusScrollBar = jScrollPane1.getVerticalScrollBar();
		}
		return jScrollPane1;
	}

	/**
	 * This method initializes bottonPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getBottonPanel() {
		if (bottonPanel == null) {
			bottonPanel = new JPanel();
			bottonPanel.setLayout(new BoxLayout(getBottonPanel(), BoxLayout.X_AXIS));
			bottonPanel.setBackground(new Color(153, 153, 255));
			bottonPanel.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.RAISED));
			bottonPanel.add(getCreateChatButton(), null);
			bottonPanel.add(getJoinChatButton(), null);
			bottonPanel.add(getRefreshButton(), null);
		}
		return bottonPanel;
	}

	public static void main (String args[]) {
		MainInterface mi = new MainInterface();
	}
	
	/**
	 * Metodo privato di inizializzazione del pannello principale
	 */
	private void initialize() {
		this.setSize(698, 330);
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setJMenuBar(getAppMenuBar());
		this.setContentPane(getMainPanel());
		this.setTitle(PeerCore.APP_NAME + " " + PeerCore.APP_VERSION);
	}
	/**
	 * Metodo privato per l'inizializzazione del pannello principale dell'applicazione
	 * @return JPanel Il pannello principale dell'applicazione
	 */
	private JPanel getMainPanel() {
		if (mainPanel == null) {
			GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
			gridBagConstraints7.insets = new Insets(1, 2, 0, 5);
			gridBagConstraints7.gridy = 2;
			gridBagConstraints7.gridx = 0;
			GridBagConstraints gridBagConstraints6 = new GridBagConstraints();
			gridBagConstraints6.fill = GridBagConstraints.BOTH;
			gridBagConstraints6.gridx = 0;
			gridBagConstraints6.gridy = 4;
			gridBagConstraints6.ipadx = 674;
			gridBagConstraints6.ipady = 82;
			gridBagConstraints6.weightx = 1.0;
			gridBagConstraints6.weighty = 1.0;
			gridBagConstraints6.insets = new Insets(0, 2, 0, 2);
			GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
			gridBagConstraints5.insets = new Insets(0, 99, 0, 99);
			gridBagConstraints5.gridy = 3;
			gridBagConstraints5.ipadx = 296;
			gridBagConstraints5.gridx = 0;
			GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
			gridBagConstraints4.insets = new Insets(2, 97, 0, 100);
			gridBagConstraints4.gridy = 0;
			gridBagConstraints4.ipadx = 312;
			gridBagConstraints4.ipady = -2;
			gridBagConstraints4.gridx = 0;
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			gridBagConstraints3.fill = GridBagConstraints.BOTH;
			gridBagConstraints3.gridx = 0;
			gridBagConstraints3.gridy = 1;
			gridBagConstraints3.ipadx = 680;
			gridBagConstraints3.ipady = 90;
			gridBagConstraints3.weightx = 1.0;
			gridBagConstraints3.weighty = 1.0;
			gridBagConstraints3.insets = new Insets(1, 1, 0, 4);
			applicationStatusLabel = new JLabel();
			applicationStatusLabel.setHorizontalAlignment(SwingConstants.CENTER);
			applicationStatusLabel.setName("applicationStatusLabel");
			applicationStatusLabel.setFont(new Font("Indigo Joker", Font.PLAIN, 14));
			applicationStatusLabel.setText("Stato dell'applicazione");
			etichetta = new JLabel();
			etichetta.setHorizontalAlignment(SwingConstants.CENTER);
			etichetta.setVerticalAlignment(SwingConstants.CENTER);
			etichetta.setFont(new Font("Indigo Joker", Font.PLAIN, 14));
			etichetta.setForeground(Color.darkGray);
			etichetta.setText("Lista chat disponibili");
			mainPanel = new JPanel();
			mainPanel.setLayout(new GridBagLayout());
			mainPanel.setBackground(new Color(195, 195, 242));
			mainPanel.add(getJScrollPane(), gridBagConstraints3);
			mainPanel.add(etichetta, gridBagConstraints4);
			mainPanel.add(applicationStatusLabel, gridBagConstraints5);
			mainPanel.add(getJScrollPane1(), gridBagConstraints6);
			mainPanel.add(getBottonPanel(), gridBagConstraints7);
		}
		return mainPanel;
	}
	
	/**
	 * Metodo privato che inizializza la barra del menu'
	 * @return JMenuBar	La barra dei menu' della finestra principale
	 */
	private JMenuBar getAppMenuBar() {
		if (barraMenu == null) {
			barraMenu = new JMenuBar();
			barraMenu.setPreferredSize(new Dimension(0, 20));
			barraMenu.setBackground(new Color(236, 175, 113));
			barraMenu.add(getAppChatMenu());
			barraMenu.add(getAppEsciMenu());
			barraMenu.add(getAppInfoMenu());
		}
		return barraMenu;
	}
	
	/**
	 * Metodo privato che inizializza il menu' 'Chat'
	 * @return JMenuBar	 Il menu' 'Chat'
	 */
	private JMenu getAppChatMenu() {
		if (chatMenu == null) {
			chatMenu = new JMenu();
			chatMenu.setText("Chat");
			chatMenu.setActionCommand("Chat");
			chatMenu.setHorizontalAlignment(SwingConstants.LEADING);
			chatMenu.setHorizontalTextPosition(SwingConstants.LEADING);
			chatMenu.add(getCreateChatItem());
			chatMenu.add(getJoinChatItem());
			chatMenu.add(getRefreshItem());
		}
		return chatMenu;
	}
	
	/**
	 * Metodo privato che inizializza il menu' 'Esci'
	 * @return JMenuBar	 Il menu' 'Esci'
	 */
	private JMenu getAppEsciMenu() {
		if (esciMenu == null) {
			esciMenu = new JMenu();
			esciMenu.setName("Esci");
			esciMenu.setHorizontalTextPosition(SwingConstants.TRAILING);
			esciMenu.setText("Esci");
		}
		return esciMenu;
	}
	
	/**
	 * Metodo privato che inializza il menu' 'Info'
	 * @return JMenuBar Il menu' 'Info'
	 */
	private JMenu getAppInfoMenu() {
		if (infoMenu == null) {
			infoMenu = new JMenu();
			infoMenu.setText("Info");
			infoMenu.setName("Info");
			infoMenu.setHorizontalAlignment(SwingConstants.LEADING);
		}
		return infoMenu;
	}
	
	/**
	 * Metodo privato che inizializza l'oggetto di menu' 'Crea chat'
	 * @return JMenuItem Elemento 'Crea chat' del menu' 'Chat'
	 */
	private JMenuItem getCreateChatItem() {
		if (createChatItem == null) {
			createChatItem = new JMenuItem();
			createChatItem.setText("Crea chat");
			createChatItem.setName("CreaChat");
		}
		return createChatItem;
	}
	
	/**
	 * Metodo privato che inizializza l'oggetto di menu' 'Partecipa alla chat'
	 * @return JMenuItem Elemento 'Partecipa alla chat' del menu' 'Chat'
	 */
	private JMenuItem getJoinChatItem() {
		if (joinChatItem == null) {
			joinChatItem = new JMenuItem();
			joinChatItem.setText("Partecipa alla chat");
			joinChatItem.setName("JoinChat");
		}
		return joinChatItem;
	}
	
	/**
	 * Metodo privato che inizializza l'oggetto di menu' 'Aggiorna la lista delle chat'
	 * @return JMenuItem Elemento 'Aggiorna la lista delle chat' del menu' 'Chat'
	 */
	private JMenuItem getRefreshItem() {
		if (refreshItem == null) {
			refreshItem = new JMenuItem();
			refreshItem.setText("Aggiorna la lista delle chat");
			refreshItem.setName("Refresh");
		}
		return refreshItem;
	}
	
	/**
	 * Metodo privato per l'inizializzazione dei Listener per gli eventi
	 * ritenuti interessati: click del mouse, selezione nella lista
	 */
	private void initializeListeners () {
		esciMenu.addMouseListener(mainListener);
		infoMenu.addMouseListener(mainListener);
		createChatItem.addMouseListener(mainListener);
		joinChatItem.addMouseListener(mainListener);
		refreshItem.addMouseListener(mainListener);
		createChatButton.addMouseListener(mainListener);
		joinChatButton.addMouseListener(mainListener);
		refreshButton.addMouseListener(mainListener);
		chatList.addMouseListener(mainListener);
		chatList.addListSelectionListener(mainListener);
		this.addWindowListener(mainListener);
	}
	
	private class AreaOutputStream extends OutputStream {
		
		private JTextArea txtArea;
		private boolean statusAppendingFlag = false;
		
		private AreaOutputStream (JTextArea tA) {
			txtArea = tA;
		}
		
		public synchronized void flush() {
			txtArea.repaint();
		}
		
		public synchronized void write(int b) {
			while (tryAppendFlag() == false) {
				try {
					wait();
				} catch (InterruptedException ie) {}
			}
			txtArea.append(new String(new byte[] {(byte)b}));
			txtArea.setCaretPosition(txtArea.getDocument().getLength());
			freeAppendFlag();
		}
		
		private synchronized boolean tryAppendFlag () {
			if (!statusAppendingFlag) {
				statusAppendingFlag = true;
				return true;
			}
			else
				return false;
		}
		private synchronized void freeAppendFlag () {
			statusAppendingFlag = false;
			notifyAll();
		}
	}
	
	/**
	 * Metodo invocato alla chiusura dell'interfaccia, che si occupa
	 * anche di cancellare il task ChatFinder
	 */
	public void close () {
		core.getChatFinder().cancel();
		this.dispose();
		System.exit(0);
	}
}