package net.jxta.chat.interfaces;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import net.jxta.chat.core.ChatFinder;
import net.jxta.chat.core.PeerCore;
import net.jxta.chat.presence.StatusServiceUnavaliableException;


final class MainInterfaceListener implements MouseListener, ListSelectionListener, WindowListener {
	
	private ChatFinder cFinder = null;
	private PeerCore peerCore = null;
	private MainInterface gui = null;
	
	/* variabile record per la selezione corrente all'interno della lista nella MainInterface gui */
	private String actualListSelection = null;
	
	protected MainInterfaceListener (MainInterface main, PeerCore core) {
		peerCore = core;
		cFinder = peerCore.getChatFinder();
		gui = main;
	}
	
	public void mouseClicked(MouseEvent e) {
		/* Se non è stato premuto il bottone sinistro */
		if (e.getButton() != MouseEvent.BUTTON1)
			return;
		
		String eventGenerator = e.getComponent().getName();
		
		/* Click sul menu' 'Info' */
		if (eventGenerator.equals("Info")) {	
			String message = PeerCore.APP_NAME + " " + PeerCore.APP_VERSION + "\n" +
			"Un programma di chat basato sul framework JXTA\n\n" +
			"Autore: Matteo Tanca [tanca@cli.di.unipi.it]";
			JOptionPane.showMessageDialog(gui, message, "Informazioni sul software", JOptionPane.INFORMATION_MESSAGE);
		}
		/* Click sul menu' 'Esci' */
		else if (eventGenerator.equals("Esci")) {
			String question = "Chiudendo la finestra principale verra' terminata ogni chat aperta\n" +
			"Si desidera proseguire?";
			int choice = JOptionPane.showOptionDialog(gui, question, null, JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE, null, null, null);
			if (choice == 0) { /* Risposta SI */
				gui.close();
			}
		}
		/* Click sul bottone 'Crea chat' */
		else if (eventGenerator.equals("CreaChat")) {
			invokeCreateChat();
		}
		/* Click sul bottone 'Aggiorna la lista' */
		else if (eventGenerator.equals("Refresh"))
			cFinder.run();
		/* Click sul bottone 'Partecipa alla chat' */
		else if (eventGenerator.equals("JoinChat"))
			invokeJoinChat();
		/* Click su un elemento della lista: memorizzazione della selezione corrente seguito dal
		 * tentativo di join in caso di doppio click
		*/
		else if (eventGenerator.equals("ChatList")) {
			if (e.getClickCount() == 2) /* Doppio click --> join su chat selezionata */
				invokeJoinChat();
		}
	}

	public void mouseEntered(MouseEvent e) {
	}

	public void mouseExited(MouseEvent e) {
	}

	public void mousePressed(MouseEvent e) {
		String eventGenerator = e.getComponent().getName();
		
		/* Click sul menu' 'Aggiorna lista delle chat' */
		if (eventGenerator.equals("Refresh"))
			cFinder.run();
		/* Click sul menu' 'Crea chat' */
		else if (eventGenerator.equals("CreaChat"))
			invokeCreateChat();
		else if (eventGenerator.equals("JoinChat"))
			invokeJoinChat();
	}

	public void mouseReleased(MouseEvent e) {
	}
	
	/**
	 * Metodo che gestisce il cambio d'elemento selezionato nella JList inserita nell'interfaccia.
	 * Alla variazione il nuovo elemento e' associato a this.actualListSelection
	 */
	public void valueChanged(ListSelectionEvent e) {
		/* Prima selezione */
		if (actualListSelection == null)
			actualListSelection = (String)((JList)e.getSource()).getSelectedValue();
		/* Selezioni successive */
		if(!e.getValueIsAdjusting()) /* La selezione e' stabile */
			actualListSelection = (String)((JList)e.getSource()).getSelectedValue();
	}
	
	/**
	 * Metodo privato per la raccolta delle informazioni necessarie all'invocazione
	 * del metodo di creazione della chat del PeerCore
	 */
	private void invokeCreateChat () {
		CreateChatDialog dialog = new CreateChatDialog(gui);
		dialog.setVisible(true);
		if (dialog.getChatName() == null || dialog.getChatDescription() == null)
			return;
		if (dialog.isInputConfirmed())
			if (dialog.getChatDescription().length() == 0 || dialog.getChatName().length() == 0) {
				JOptionPane.showMessageDialog(gui, "E' necessario immettere sia il nome sia la descrizione della chat", "Informazioni mancanti", JOptionPane.ERROR_MESSAGE);
				return;
			}
			peerCore.createStandardChat(dialog.getChatName(), dialog.getChatDescription());
	}
	
	/**
	 * Metodo privato che si occupa di effettuare l'operazione di joinGroup al PeerGroup
	 * corrispondente alla selezione corrente (actualListSelection della MainInterface gui),
	 * mediante invocazione del metodo corrispondente nel nucleo PeerCore
	 */
	private void invokeJoinChat () {
		if (actualListSelection == null) {
			JOptionPane.showMessageDialog(gui, "Per partecipare a una chat e' necessario selezionarla dalla lista", "Richiesta di selezione", JOptionPane.INFORMATION_MESSAGE);
			return;
		}
		if(peerCore.isInChat(actualListSelection)) {
			System.out.println("Si e' gia' nella stanza di chat selezionata");
			return;
		}
		String nick = JOptionPane.showInputDialog(gui, "Inserisci il nick da utilizzare", "JXTA_chat_user");
		/* Qualora non venga premuto OK */
		if (nick == null)
			return;
		try {
			peerCore.joinChat(actualListSelection, nick);
		} catch (StatusServiceUnavaliableException e) {
			System.out.println("Impossibile completare la join");
			e.printStackTrace();
		}
		
	}

	public void windowActivated(WindowEvent e) {
	}

	public void windowClosed(WindowEvent e) {
	}

	public void windowClosing(WindowEvent e) {
		gui.close();
	}

	public void windowDeactivated(WindowEvent e) {
	}

	public void windowDeiconified(WindowEvent e) {
	}

	public void windowIconified(WindowEvent e) {
	}

	public void windowOpened(WindowEvent e) {
	}
}
