package net.jxta.chat.presence;
import net.jxta.service.Service;

/**
 * @author Matteo Tanca [tanca@cli.di.unipi.it]
 * @version 0.1
 * Interfaccia del servizio informativo sullo stato degli utenti. Permette agli utenti di
 * scambiare fra loro informazioni quali lo stato ed il nick, aggiornabili dinamicamente
 * mediante pubblicazione di UserStatusAdvertisement
 */
public interface StatusService extends Service {
    
	/* Stringa rappresentazione del ModuleClassID del servizio */
    public static final String refModuleClassID = "urn:jxta:uuid-59A9A948905341119EAB8630EED42AB905";
    /* Informazioni utili alla pubblicazione dell'advertisement */
    public static final String SERVICE_NAME = "Users Status Service";
    public static final String SERVICE_DESCRIPTION = "Servizio informativo sullo stato degli " +
    										"utenti della chat (Peergroup) ad esso associata";

    /* Stati utente contemplati */
    public final int OFFLINE_STATUS = 0;
	public final int ONLINE_STATUS = 1;
	public final int BUSY_STATUS = 2;
	public final int AWAY_STATUS = 3;
	public final int BACK_SOON_STATUS = 4;		

    /**
     * Metodo per l'aggiunta di un listener al servizio, a cui verranno notificati eventuali
     * messaggi
     * @param  UsersStatusListener
     */
    public void addListener (UsersStatusListener l);
    
    /**
     * Metodo per la rimozione dal servizio di un listener, che non sara' piu' avvisato
     * qualora dovessero giungere dei messaggi
     * @param UsersStatusListener
     */
    public boolean removeListener (UsersStatusListener listener);

    /**
     * Metodo per la pubblicazione delle informazioni relative ad un utente
     * @param userStatus L'attuale stato dell'utente di cui si stanno pubblicando le informazioni
     * @param nickname Il nome utilizzato dall'utente
     */
    public void publishUserInfo (int userStatus, String nickname);

    /**
     * Metodo che invia una query per trovare le informazioni relative all'utente identificato
     * dall'argomento. Ogni risposta sara' inviata agli UsersStatusListener in ascolto
     * sul servizio
     * @param peerID Il peerID da utilizzare per trovare le informazioni
     */
    public void findUserInfo (String peerID);
}
